<?php

namespace Drupal\eudonet;

/**
 * Represents properties of Eudonet Objects.
 *
 * @property-read mixed Value
 * @property-read mixed DbValue
 */
class EudonetSearchFieldItem {

  public function __construct(array $data = []) {
    $this->data = array_merge([
      'DbValue' => '',
      'Value' => '',
    ], $data);

    $this->validate();
  }

  public function __get(string $name) {
    if ($name === 'data') {
      throw new \InvalidArgumentException("Attempt to set illegal property: $name");
    }

    if (in_array($name, ['Value', 'DbValue'])) {
      return $this->data[$name];
    }

    throw new \InvalidArgumentException("Unexpected attempt to get the property: $name");
  }

  public function __set(string $name, mixed $value): void {
    if ($name === 'data') {
      $this->data = $value;
      return;
    }

    if ($name === 'Value') {
      $this->setValue($value);
      return;
    }

    throw new \InvalidArgumentException("Unexpected attempt to set the property: $name");
  }

  public function __toString(): string {
    return $this->getValue();
  }

  public function toArray(): string {
    return [
      'DescId' => $this->id(),
      'Value' => $this->getValue(),
    ];
  }

  private function validate(): void {
    foreach ([
      'DbValue',
      'DescId',
      'Value',
    ] as $key) {
      if (!isset($this->data[$key])) {
        throw new \ValueError("Missing required property: $key");
      }
    }

    if (!is_numeric($this->data['DescId'])) {
      throw new \ValueError("Invalid DescId");
    }
  }

  /**
   * Get the unique Field ID.
   *
   * @return int
   *   Desc ID.
   */
  public function id(): int {
    return $this->data['DescId'];
  }

  /**
   * Get the value of the field as it is stored in the database.
   *
   * @return mixed
   *   Field value.
   */
  public function getValue(): mixed {
    return $this->data['DbValue'];
  }

  /**
   * Get the value of the field for display purposes.
   *
   * @return mixed
   *   Field value.
   */
  public function getDisplayValue(): mixed {
    return $this->data['Value'];
  }

  /**
   * Get the raw value of the field as it appears in Eudonet API.
   *
   * @return array
   *   Field value.
   */
  public function getRawValue(): array {
    return $this->data;
  }

  /**
   * Set the value of the field.
   *
   * This also resets the display value.
   *
   * @param int|string|bool|null $value
   *   The value.
   */
  public function setValue(null|int|string|bool $value): void {
    // Setting the value resets the display value.
    $this->data['DbValue'] = $this->data['Value'] = $value;
  }

  /**
   * Whether the field is empty.
   *
   * @return bool
   *   True or False.
   */
  public function isEmpty(): bool {
    return empty($this->getValue());
  }

}
