<?php

namespace Drupal\eudonet;

use Drupal\eudonet\Traits\EudonetMappingTrait;

/**
 * Eudonet Search Result Item.
 *
 * @package Drupal\eudonet
 */
class EudonetSearchResultItem {

  use EudonetMappingTrait;

  /**
   * Create a Query Result Item.
   *
   * @param array $item
   *   The item object from response.
   * @param string $mapping
   *   The mapping Plugin ID.
   *
   * @todo Currently, 'Fields' are searched and retrieved on the fly using
   *   their DescId. Ideally, key the fields by Field ID so that searching
   *   can be performed more efficiently. Possibly, create FieldItem objects
   *   for all fields in the constructor so that the field values can later
   *   be set like we do on a regular PHP object. The values can then be
   *   used to create/update the relevant model on Eudonet.
   */
  public function __construct(
    protected array $item,
    string $mapping = 'default',
  ) {
    $this->setMapping($mapping);
    $this->ensureMapping();
  }

  /**
   * Get field value, if present.
   *
   * @param string $name
   *   The Desc ID (Field ID) or a valid, mapped key.
   *
   * @return \Drupal\eudonet\EudonetSearchFieldItem|null
   *   Field value.
   */
  public function __get(string $name): ?EudonetSearchFieldItem {
    $desc_id = $this->getFieldId($name);

    $path = self::getDescIdPath($desc_id, $this->item, []);
    if (empty($path)) {
      return NULL;
    }

    return self::getDescIdItem($this->item, $path);
  }

  /**
   * Get the item id.
   *
   * @return int
   *   The file id for this item.
   */
  public function id(): int {
    return $this->item['FileId'];
  }

  /**
   * Get an array representation of the result item.
   *
   * Experimental: Use for debugging only.
   *
   * @return array
   *   An array representing the item.
   */
  public function toArray(): array {
    $result['id'] = $this->id();

    foreach ($this->getMappedFieldNames() as $name) {
      if (!$fieldItem = $this->$name) {
        continue;
      }

      $result[$name] = $fieldItem->getValue();
    }

    return $result;
  }

  /**
   * Recursively get the desc_id parent path.
   *
   * @param int $desc_id
   *   The desc id to look for.
   * @param array $tree
   *   A tree representing an eudonet query result item.
   * @param array $path
   *   The path to the desc id.
   *
   * @return array|null
   *   The desc id parent path or NULL.
   */
  private static function getDescIdPath(int $desc_id, array $tree, array $path): ?array {
    $found = NULL;
    foreach ($tree as $key => $item) {
      if ($key === 'DescId' && $item === $desc_id) {
        return $path;
      }
      elseif (is_array($item) && $found === NULL) {
        $found = self::getDescIdPath($desc_id, $item, array_merge($path, [$key]));
      }
    }

    return $found;
  }

  /**
   * Get the item value according to a Desc ID.
   *
   * @param array $tree
   *   A tree representing an eudonet query result item.
   * @param array $path
   *   The path to the desc id.
   * @param int $depth
   *   The tree depth.
   *
   * @return \Drupal\eudonet\EudonetSearchFieldItem|null
   *   Field data, if found; NULL otherwise.
   */
  private static function getDescIdItem(array $tree, array $path, int $depth = 0): ?EudonetSearchFieldItem {
    $max_depth = count($path);
    $found = NULL;
    foreach ($tree as $key => $item) {
      if ($depth == $max_depth - 1 && isset($path[$depth]) && $key == $path[$depth]) {
        return new EudonetSearchFieldItem($item);
      }
      elseif (is_array($item) && $depth < $max_depth && $found === NULL) {
        $found = self::getDescIdItem($item, $path, $depth + 1);
      }
    }
    return $found;
  }

}
