<?php

namespace Drupal\eudonet\Form;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\State\StateInterface;
use Drupal\eudonet\Eudonet;
use Drupal\eudonet\Exception\AuthException;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Eudonet API Settings Form.
 */
class SettingsForm extends ConfigFormBase {

  use MessengerTrait;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    protected StateInterface $state,
    protected Eudonet $eudonet,
  ) {
    parent::__construct($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('eudonet'),
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      $this->eudonet::EUDONET_CONFIG,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'eudonet_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config($this->eudonet::EUDONET_CONFIG);

    $form['base_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Base URL'),
      '#description' => $this->t('Example: @example', [
        '@example' => 'https://xrm.eudonet.com/EudoAPI/',
      ]),
      '#default_value' => $config->get('base_url'),
    ];

    $auth = $config->get('authentication');
    $form['authentication'] = [
      '#type' => 'details',
      '#title' => $this->t('Authentication'),
      '#description' => $this->t('Server authentication.'),
      '#open' => empty($auth),
      '#tree' => TRUE,
    ];
    $form['authentication']['subscriber_login'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subscriber login'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $auth['subscriber_login'] ?? '',
      '#required' => TRUE,
    ];
    $form['authentication']['subscriber_password'] = [
      '#type' => 'password',
      '#title' => $this->t('Subscriber password'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $auth['subscriber_password'] ?? '',
      '#description' => $this->t('Leave blank to make no changes.'),
    ];
    $form['authentication']['base_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Database name'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $auth['base_name'] ?? '',
      '#required' => TRUE,
    ];
    $form['authentication']['user_login'] = [
      '#type' => 'textfield',
      '#title' => $this->t('User login'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $auth['user_login'] ?? '',
      '#required' => TRUE,
    ];
    $form['authentication']['user_password'] = [
      '#type' => 'password',
      '#title' => $this->t('User password'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $auth['user_password'] ?? '',
      '#description' => $this->t('Leave blank to make no changes.'),
    ];
    $form['authentication']['language'] = [
      '#type' => 'select',
      '#title' => $this->t('Language'),
      '#options' => [
        // @todo Use an Enum.
        'LANG_00' => $this->t('French'),
        'LANG_01' => $this->t('English'),
        'LANG_02' => $this->t('German'),
        'LANG_03' => $this->t('Dutch'),
        'LANG_04' => $this->t('Spanish'),
        'LANG_05' => $this->t('Italian'),
      ],
      '#multiple' => FALSE,
      '#default_value' => $auth['language'] ?? 'LANG_00',
    ];
    $form['authentication']['product_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Product name'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $auth['product_name'] ?? '',
    ];

    // Display info about the current auth token.
    $authToken = $this->state->get('eudonet.auth_token');
    $form['current_token'] = [
      '#type' => 'details',
      '#title' => $this->t('Current token'),
      '#description' => $this->t('Click on <em>Test credentials</em> to test the authentication credentials.'),
      '#open' => empty($authToken),
    ];
    $form['current_token']['value'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Value'),
      '#attributes' => [
        'disabled' => 'disabled',
      ],
      '#default_value' => $authToken?->getValue(),
      '#access' => !empty($authToken),
    ];
    $form['current_token']['expires_at'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Expiration'),
      '#attributes' => [
        'disabled' => 'disabled',
      ],
      '#default_value' => $authToken?->getExpiresAt()->format('c'),
      '#access' => !empty($authToken),
    ];
    $form['current_token']['created_at'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Created at'),
      '#description' => $this->t('The time on Eudonet server at which the token was generated.'),
      '#attributes' => [
        'disabled' => 'disabled',
      ],
      '#default_value' => $authToken?->getCreatedAt()->format('c'),
      '#access' => !empty($authToken),
    ];
    $form['current_token']['actions'] = [
      '#type' => 'actions',
    ];
    $form['current_token']['actions']['refresh_token'] = [
      '#type' => 'submit',
      '#value' => $this->t('Test credentials'),
      '#submit' => ['::refreshToken'],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config($this->eudonet::EUDONET_CONFIG);

    // Manage password elements. For security reasons, we do not want to
    // re-expose the password to the user.
    $old_auth = $config->get('authentication');
    $authentication = $form_state->getValue('authentication');

    if (empty($authentication['subscriber_password'])) {
      $authentication['subscriber_password'] = $old_auth['subscriber_password'];
    }

    if (empty($authentication['user_password'])) {
      $authentication['user_password'] = $old_auth['user_password'];
    }

    $config
      ->set('base_url', $form_state->getValue('base_url'))
      ->set('authentication', $authentication)
      ->save();

    // Clear the old token generated by old credentials.
    $this->state->delete('eudonet.auth_token');

    parent::submitForm($form, $form_state);
  }

  /**
   * Make an authentication request to test the authentication to the API.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function refreshToken(array &$form, FormStateInterface $form_state) {
    try {
      $this->eudonet->getAuthToken();
      $this->messenger()->addStatus($this->t('Authentication succeeded.'));
    }
    catch (AuthException $ex) {
      $this->messenger()->addError($ex->getMessage());
    }
  }

}
