<?php

namespace Drupal\eudonet\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\eudonet\Eudonet;
use Drupal\eudonet\Traits\EudonetMappingTrait;
use GuzzleHttp\Psr7\Response;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for Eudonet query result plugins.
 */
abstract class EudonetQueryResultBase extends PluginBase implements EudonetQueryResultInterface, ContainerFactoryPluginInterface {

  use EudonetMappingTrait;

  /**
   * The guzzle response object.
   *
   * @var \GuzzleHttp\Psr7\Response|null
   */
  protected ?Response $guzzleResponse;

  /**
   * The response array from the Eudonet API.
   *
   * @var array
   */
  protected array $response;

  /**
   * The guzzle response header.
   *
   * @var array
   */
  protected array $headers;

  /**
   * Mechanize to ensure the authentication when needed.
   *
   * @var bool
   */
  protected bool $firstTry;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    /**
     * The Eudonet Connector.
     *
     * @var \Drupal\eudonet\Eudonet
     */
    protected Eudonet $eudonet
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->guzzleResponse = $this->configuration['response'];
    $this->response =
      Json::decode($this->guzzleResponse->getBody()->getContents());
    $this->headers = $this->guzzleResponse->getHeaders();
    $this->firstTry = TRUE;
  }

  /**
   * Creates an instance of the plugin.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container to pull out services used in the plugin.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of this plugin.
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $configuration, $plugin_id, $plugin_definition,
      $container->get('eudonet')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function success(): bool {
    return boolval($this->response['ResultInfos']['Success']);
  }

  /**
   * Whether the request failed due to an error in the authentication token.
   *
   * @return bool
   *   True or False.
   */
  public function isTokenError(): bool {
    return in_array($this->getErrorNumber(), Eudonet::TOKEN_ERRORS);
  }

  /**
   * {@inheritdoc}
   */
  public function ensureAuth(): void {
    if (!$this->firstTry || !$this->isTokenError()) {
      return;
    }

    $this->firstTry = FALSE;
    $this->eudonet->getAuthToken();
  }

  /**
   * {@inheritdoc}
   */
  public function getApiMessage(): string {
    return $this->response['ResultInfos']['ApiMessage'];
  }

  /**
   * {@inheritdoc}
   */
  public function getErrorMessage(): string {
    return $this->response['ResultInfos']['ErrorMessage'];
  }

  /**
   * {@inheritdoc}
   */
  public function getErrorNumber(): int {
    return intval($this->response['ResultInfos']['ErrorNumber']);
  }

  /**
   * {@inheritdoc}
   */
  public function getGuzzleResponse(): Response {
    return $this->guzzleResponse;
  }

  /**
   * {@inheritdoc}
   */
  public function getResponse(): array {
    return $this->response;
  }

  /**
   * {@inheritdoc}
   */
  public function getMaxAllowedRequests(): int {
    return intval($this->headers['X-CALL-MAX'][0]);
  }

  /**
   * {@inheritdoc}
   */
  public function getRemainingCallsByIp(): int {
    return intval($this->headers['X-CALL-BYIP-REMAIN'][0]);
  }

  /**
   * {@inheritdoc}
   */
  public function getRemainingCalls(): int {
    return intval($this->headers['X-CALL-REMAIN'][0]);
  }

  /**
   * {@inheritdoc}
   */
  public function isQuotaExceeded(): bool {
    return $this->getRemainingCalls() === 0 && $this->getRemainingCallsByIp() === 0;
  }

}
