<?php

namespace Drupal\eudonet\Plugin;

use Drupal\Component\Plugin\PluginInspectionInterface;
use GuzzleHttp\Psr7\Response;

/**
 * An interface for Eudonet Query Result plugins.
 */
interface EudonetQueryResultInterface extends PluginInspectionInterface {

  /**
   * Get success info.
   *
   * @return bool
   *   True or False.
   */
  public function success(): bool;

  /**
   * Make an auth request if the first request has failed (TOKEN_ERRORS).
   *
   * @todo Consider removing this method.
   *   The app using the API should ensure auth before making a request.
   */
  public function ensureAuth(): void;

  /**
   * Get the api response message.
   *
   * @return string
   *   The message returned by the API response.
   */
  public function getApiMessage(): string;

  /**
   * Get the error message.
   *
   * @return string
   *   May be empty for a successful request.
   */
  public function getErrorMessage(): string;

  /**
   * Get the error number.
   *
   * @return int
   *   May be empty for a successful request.
   */
  public function getErrorNumber(): int;

  /**
   * Get the raw response object.
   *
   * @return \GuzzleHttp\Psr7\Response
   *   A raw response object from the guzzle request.
   */
  public function getGuzzleResponse(): Response;

  /**
   * Get Eudonet API response data.
   *
   * @return array
   *   An array representing the Eudonet API response content.
   */
  public function getResponse(): array;

  /**
   * Header "x-call-max" => Maximum number of requests allowed.
   *
   * @return int
   *   The maximum number of requests allowed.
   */
  public function getMaxAllowedRequests(): int;

  /**
   * Header "x-call-byip-remain" => Remaining number of calls for the same ip.
   *
   * @return int
   *   The remaining number of calls for the same ip.
   */
  public function getRemainingCallsByIp(): int;

  /**
   * Header x-call-remain => Number of calls remaining.
   *
   * @return int
   *   The number of calls remaining.
   */
  public function getRemainingCalls(): int;

  /**
   * Whether the API request quota has been reached.
   *
   * @return bool
   *   True if new requests cannot be sent.
   */
  public function isQuotaExceeded(): bool;

}
