<?php

namespace Drupal\eudonet\Traits;

use Drupal\eudonet\EudonetQueryCondition;

/**
 * Trait EudonetQueryConditionTrait.
 *
 * @package Drupal\eudonet\Traits
 */
trait EudonetQueryConditionTrait {

  /**
   * An array containing all conditions.
   *
   * @var \Drupal\eudonet\EudonetQueryCondition[]
   */
  public array $conditions = [];

  /**
   * Filter the query with a condition.
   *
   * @param string|int $field
   *   Mapped field name or the Field ID.
   * @param string $value
   *   The field value.
   * @param string $operator
   *   The operator used to filter the field.
   *
   * @return $this
   */
  public function condition(string|int $field, string $value, string $operator = '='): static {
    $iop = 'None';
    if (count($this->conditions) > 0) {
      $iop = (!empty($this->interOperator)) ? $this->interOperator : 'AND';
    }
    $this->conditions[] = new EudonetQueryCondition($field, $value, $operator, $iop, $this->mappingId);

    return $this;
  }

  /**
   * Add condition: Field is not empty.
   *
   * @param string|int $field
   *   Mapped field name or the Field ID.
   *
   * @return $this
   */
  public function exists(string|int $field): static {
    return $this->condition($field, '', 'NOT EMPTY');
  }

  /**
   * Add condition: Field is empty.
   *
   * @param string|int $field
   *   Mapped field name or the Field ID.
   *
   * @return $this
   */
  public function notExists(string|int $field): static {
    return $this->condition($field, '', 'EMPTY');
  }

  /**
   * Add a group condition.
   *
   * @param \Drupal\eudonet\EudonetQueryCondition $condition
   *   A condition object representing the condition group.
   *
   * @return $this
   */
  public function conditionGroup(EudonetQueryCondition $condition): static {
    $this->conditions[] = $condition;

    return $this;
  }

  /**
   * Create a 'OR' condition group.
   *
   * @return \Drupal\eudonet\EudonetQueryCondition
   *   A condition group with OR conjunction.
   */
  public function orGroup(): EudonetQueryCondition {
    return $this->group('OR');
  }

  /**
   * Create a 'AND' condition group.
   *
   * @return \Drupal\eudonet\EudonetQueryCondition
   *   A condition group with AND conjunction.
   */
  public function andGroup(): EudonetQueryCondition {
    return $this->group('AND');
  }

  /**
   * Create a condition group according to the Eudonet API inter operators.
   *
   * @param string $inter_operator
   *   An inter operator. See EudonetHelper::INTER_OPERATORS.
   *
   * @return \Drupal\eudonet\EudonetQueryCondition
   *   A condition object.
   */
  protected function group(string $inter_operator): EudonetQueryCondition {
    return new EudonetQueryCondition(NULL, NULL, NULL, $inter_operator, $this->mappingId);
  }

}
